import { switchMap } from 'rxjs/operators';
import { createSseSelector, } from '../selectors/sse-selector';
/**
 * An operator function for handling Server-Sent Events (SSE) in an RxJS pipeline.
 *
 * @template R - The type of the parsed data from the SSE.
 * @template T - The type of the input `Response` object, defaults to `Response`.
 *
 * @param options - Configuration options for the SSE selector.
 *
 * @returns An `OperatorFunction` that transforms a stream of `Response` objects
 * into a stream of `ServerSentEvent` objects containing parsed data of type `R`.
 *
 * @example
 * ```typescript
 * import { sseMap } from '@equinor/fusion-framework-module-http/operators';
 * import { fromFetch } from 'rxjs/fetch';
 *
 * const response$ = fromFetch('https://example.com/sse', {
 *   method: 'GET',
 *   headers: {
 *     'Accept': 'text/event-stream',
 *   },
 * }).pipe(
 *   sseMap( { /* SSE selector options *\/ })
 * ).subscribe((event) => {
 *   console.log(event.data); // Process the parsed SSE data
 * });
 * ```
 */
export const sseMap = (options) => (source) => source.pipe(switchMap(createSseSelector(options)));
export default sseMap;
//# sourceMappingURL=sse.operator.js.map