/* eslint-disable @typescript-eslint/no-explicit-any */
import { HttpClientMsal } from './lib/client';
import { HttpClientConfigurator, } from './configurator';
import { HttpClientProvider } from './provider';
/**
 * HTTP module with MSAL authentication.
 */
export const module = {
    name: 'http',
    /**
     * Configures the HTTP module with MSAL authentication.
     *
     * This function creates a new `HttpClientConfigurator` instance using the `HttpClientMsal` class.
     * The `HttpClientConfigurator` is responsible for configuring the HTTP client with the necessary options,
     * such as the base URL, request headers, and other settings.
     *
     * @returns A new `HttpClientConfigurator` instance configured for MSAL authentication.
     */
    configure: () => new HttpClientConfigurator(HttpClientMsal),
    /**
     * Initializes the HTTP client provider with MSAL authentication.
     *
     * This function is responsible for setting up the default HTTP request handler
     * to acquire an access token from MSAL and attach it to the request headers
     * when the request includes scopes.
     *
     * @param config - The module configuration.
     * @param hasModule - A function to check if a module is available.
     * @param requireInstance - A function to get an instance of a module.
     * @returns A promise that resolves to the HTTP client provider.
     */
    initialize: async ({ config, hasModule, requireInstance, }) => {
        const httpProvider = new HttpClientProvider(config);
        if (hasModule('auth')) {
            const authProvider = await requireInstance('auth');
            httpProvider.defaultHttpRequestHandler.set('MSAL', async (request) => {
                const { scopes = [] } = request;
                if (scopes.length) {
                    /** TODO should be try catch, check caller for handling */
                    const token = await authProvider.acquireToken({
                        scopes,
                    });
                    if (token) {
                        const headers = new Headers(request.headers);
                        headers.set('Authorization', `Bearer ${token.accessToken}`);
                        return { ...request, headers };
                    }
                }
            });
        }
        return httpProvider;
    },
};
/**
 * Configures the HTTP module with MSAL authentication.
 */
export const configureHttp = (configure) => ({
    module,
    configure,
});
/**
 * Configures the HTTP client with MSAL authentication.
 *
 * This function creates a module configurator that can be used to configure the HTTP module
 * with MSAL authentication. The configurator takes a name and a set of HTTP client options,
 * and returns a module configurator that can be used to configure the HTTP module.
 *
 * @param name - The name of the HTTP client configuration.
 * @param args - The HTTP client options, including the MSAL configuration.
 * @returns A module configurator that can be used to configure the HTTP module.
 */
export const configureHttpClient = (name, args) => ({
    module,
    configure: (config) => {
        config.configureClient(name, args);
    },
});
export default module;
//# sourceMappingURL=module.js.map