/**
 * Represents an error that occurs when handling an HTTP response.
 * @template TResponse The type of the HTTP response.
 */
export declare class HttpResponseError<TResponse = Response> extends Error {
    readonly response: TResponse;
    static Name: string;
    constructor(message: string, response: TResponse, options?: ErrorOptions);
}
/**
 * Represents an error that occurs when handling a JSON response in an HTTP request.
 * Extends the base `HttpResponseError` class.
 *
 * @template TType - The type of the data associated with the error.
 * @template TResponse - The type of the HTTP response.
 */
export declare class HttpJsonResponseError<TType = unknown, TResponse = Response> extends HttpResponseError<TResponse> {
    static Name: string;
    readonly data?: TType;
    /**
     * Creates a new instance of `HttpJsonResponseError`.
     *
     * @param message - The error message.
     * @param response - The HTTP response associated with the error.
     * @param options - Additional options for the error, including the associated data.
     */
    constructor(message: string, response: TResponse, options?: ErrorOptions & {
        data?: TType;
    });
}
/**
 * Represents an error that occurs when handling a server-sent event (SSE) HTTP response.
 *
 * @template TType - The type of additional data associated with the error.
 * @template TResponse - The type of the HTTP response object.
 *
 * @extends HttpResponseError<TResponse>
 */
export declare class ServerSentEventResponseError<TType = unknown, TResponse = Response> extends HttpResponseError<TResponse> {
    static Name: string;
    /**
     * Creates a new instance of the error.
     *
     * @param message - The error message describing the cause of the error.
     * @param response - The HTTP response associated with the error.
     * @param options - Optional error options, which may include additional data of type `TType`.
     */
    constructor(message: string, response: TResponse, options?: ErrorOptions & {
        data?: TType;
    });
}
