import { Subject } from 'rxjs';
import type { Observable, ObservableInput } from 'rxjs';
import type { IHttpRequestHandler, IHttpResponseHandler } from '../operators';
import type { BlobResult, FetchRequest, FetchRequestInit, FetchResponse, IHttpClient, JsonRequest, StreamResponse } from './types';
import { type ServerSentEvent, type SseSelectorOptions } from '../selectors/sse-selector';
/**
 * Configuration options for creating an `HttpClient` instance.
 *
 * @template TRequest - The type of the request object. Defaults to `FetchRequest`.
 * @template TResponse - The type of the response object. Defaults to `Response`.
 * @property {IHttpRequestHandler<TRequest>} requestHandler - A handler for customizing the request before it is sent.
 * @property {IHttpResponseHandler<TResponse>} responseHandler - A handler for customizing the response after it is received.
 */
export type HttpClientCreateOptions<TRequest extends FetchRequest = FetchRequest, TResponse = Response> = {
    requestHandler: IHttpRequestHandler<TRequest>;
    responseHandler: IHttpResponseHandler<TResponse>;
};
/** Base http client for executing requests */
export declare class HttpClient<TRequest extends FetchRequest = FetchRequest, TResponse extends FetchResponse = FetchResponse> implements IHttpClient<TRequest, TResponse> {
    uri: string;
    /**
     * A request handler that can be used to customize the request before it is sent.
     * This property is part of the `HttpClientCreateOptions` configuration object used to create an `HttpClient` instance.
     */
    readonly requestHandler: IHttpRequestHandler<TRequest>;
    /**
     * A handler for customizing the response after it is received.
     * This property is part of the `HttpClientCreateOptions` configuration object used to create an `HttpClient` instance.
     */
    readonly responseHandler: IHttpResponseHandler<TResponse>;
    /**
     * A stream of requests that are about to be executed.
     * This property is used internally by the `HttpClient` class to manage the lifecycle of requests.
     */
    protected _request$: Subject<TRequest>;
    /**
     * A stream of responses that have been received.
     * This property is used internally by the `HttpClient` class to manage the lifecycle of responses.
     */
    protected _response$: Subject<TResponse>;
    /**
     * A stream that is used to signal the abortion of requests.
     * This property is used internally by the `HttpClient` class to manage the lifecycle of requests.
     */
    protected _abort$: Subject<void>;
    /**
     * A stream of requests that are about to be executed.
     */
    get request$(): Observable<TRequest>;
    /**
     * A stream of responses that have been received.
     */
    get response$(): Observable<TResponse>;
    constructor(uri: string, options?: Partial<HttpClientCreateOptions<TRequest, TResponse>>);
    /**
     * Internal method called by child classes to perform initialization logic in the constructor.
     * This method is intended to be overridden by child classes to add their own initialization logic.
     * @protected
     * @virtual
     */
    protected _init(): void;
    /**
     * Fetches data from the specified path and returns a stream response.
     *
     * @param path - The path to fetch data from.
     * @param args - Optional request initialization options, including a custom selector function.
     * @returns A stream response containing the fetched data.
     */
    fetch$<T = TResponse>(path: string, args?: FetchRequestInit<T, TRequest, TResponse>): StreamResponse<T>;
    /**
     * Fetches data from the specified path and returns a Promise containing the fetched data.
     *
     * @param path - The path to fetch data from.
     * @param args - Optional request initialization options, including a custom selector function.
     * @returns A Promise containing the fetched data.
     */
    fetch<T = TResponse>(path: string, args?: FetchRequestInit<T, TRequest, TResponse>): Promise<T>;
    /** @deprecated */
    fetchAsync<T = TResponse>(path: string, args?: FetchRequestInit<T, TRequest, TResponse>): Promise<T>;
    /**
     * Fetches data from the specified path and returns a stream response containing the data in JSON format.
     *
     * @param path - The path to fetch the data from.
     * @param args - Optional request initialization options, including a custom selector function and request body.
     *   - `body`: The request body, which will be automatically serialized to JSON if it's an object.
     *   - `selector`: A custom selector function to transform the response data. If not provided, the `jsonSelector` function will be used.
     *   - `headers`: Additional headers to include in the request. The `Accept` and `Content-Type` headers will be automatically set to `application/json`.
     * @returns A stream response containing the fetched data in JSON format.
     */
    json$<T = unknown>(path: string, args?: FetchRequestInit<T, JsonRequest<TRequest>, TResponse>): StreamResponse<T>;
    /**
     * Fetches data from the specified path and returns a Promise containing the fetched data in JSON format.
     *
     * @param path - The path to fetch the data from.
     * @param args - Optional request initialization options, including a custom selector function and request body.
     *   - `body`: The request body, which will be automatically serialized to JSON if it's an object.
     *   - `selector`: A custom selector function to transform the response data. If not provided, the `jsonSelector` function will be used.
     *   - `headers`: Additional headers to include in the request. The `Accept` and `Content-Type` headers will be automatically set to `application/json`.
     * @returns A Promise containing the fetched data in JSON format.
     */
    json<T = unknown>(path: string, args?: FetchRequestInit<T, JsonRequest<TRequest>, TResponse>): Promise<T>;
    /**
     * Fetches a blob resource from the specified path and returns a stream response.
     *
     * @param path - The path to the blob resource.
     * @param args - Optional request initialization options, including a custom selector function.
     * @returns A stream response containing the fetched blob data.
     */
    blob$<T = BlobResult>(path: string, args?: FetchRequestInit<T, TRequest, TResponse>): StreamResponse<T>;
    /**
     * Fetches a blob from the specified path and returns a Promise that resolves to the blob result.
     *
     * @param path - The path to fetch the blob from.
     * @param args - Optional arguments for the fetch request, including request body, headers, and response type.
     * @returns A Promise that resolves to the blob result.
     */
    blob<T = BlobResult>(path: string, args?: FetchRequestInit<T, TRequest, TResponse>): Promise<T>;
    /**
     * Initiates a Server-Sent Events (SSE) stream request to the specified path and returns a stream of events.
     *
     * @template T - The type of the event data expected from the SSE stream.
     * @param path - The endpoint path to connect to for the SSE stream.
     * @param args - Optional fetch request initialization options, including headers and abort signal.
     * @param options - Optional selector options for customizing the SSE stream, excluding the abort signal.
     *
     * @returns A `StreamResponse` that emits `ServerSentEvent<T>` objects as they are received from the server.
     *
     * @example
     * const sse$ = httpClient.sse(
     *  '/events',
     *  { method: 'POST', body: JSON.stringify({ prompt: 'tell me a joke' }) },
     *  { eventFilter: ['message'] }
     * );
     * sse$.subscribe({
     *  next: (event) => console.log(event),
     *  error: (err) => console.error(err),
     *  complete: () => console.log('Completed'),
     * });
     */
    sse$<T = unknown>(path: string, args?: FetchRequestInit<ServerSentEvent<T>, TRequest, TResponse> | null, options?: Omit<SseSelectorOptions<T>, 'abortSignal'>): StreamResponse<ServerSentEvent<T>>;
    /** @deprecated */
    jsonAsync<T = unknown>(path: string, args?: FetchRequestInit<T, JsonRequest<TRequest>, TResponse>): Promise<T>;
    /**
     * Executes an HTTP request using the specified method and path.
     *
     * @param method - The HTTP method to use for the request, such as 'fetch', 'json', or 'blob'.
     * @param path - The path to the resource to fetch.
     * @param init - Optional request initialization options, including request body, headers, and response type.
     * @returns The result of the HTTP request, which will be of the same type as the return value of the specified method.
     */
    execute<T = TResponse, TMethod extends 'fetch' | 'fetch$' | 'json' | 'json$' = 'fetch'>(method: TMethod, path: string, init?: FetchRequestInit<T, TRequest, TResponse>): ReturnType<IHttpClient[TMethod]>;
    /**
     * Aborts any ongoing HTTP requests made by this `IHttpClient` instance.
     * This will trigger the `takeUntil` operator in the `_fetch$` method,
     * causing any in-flight requests to be cancelled.
     */
    abort(): void;
    /**
     * Fetches data from the specified path and returns an Observable that emits the response.
     *
     * @param path - The path to fetch the data from.
     * @param args - Optional arguments for the fetch request, including a response selector function, request body, headers, and response type.
     * @returns {Observable<T>} An Observable that emits the response data.
     *
     * This method handles the following steps:
     * 1. Resolves the full URL by combining the base URI and the provided path.
     * 2. Prepares the request by passing it through the `requestHandler.process()` method.
     * 3. Executes the fetch request using the prepared request.
     * 4. Prepares the response by passing it through the `responseHandler.process()` method.
     * 5. Applies the optional response selector function to transform the response data.
     * 6. Cancels the request if the `_abort$` observable emits.
     */
    protected _fetch$<T = TResponse>(path: string, args?: FetchRequestInit<T, TRequest, TResponse>): Observable<T>;
    /**
     * Prepares the request by passing it through the `requestHandler.process()` method.
     * This method is an implementation detail of the `_fetch$()` method, and is not part of the public API.
     * It takes a `TRequest` object as input, which represents the request data, and returns an `ObservableInput<TRequest>`,
     * which can be used to further process the request before it is executed.
     *
     * @param init The request data to be processed.
     * @returns An `ObservableInput<TRequest>` that represents the processed request.
     */
    protected _prepareRequest(init: TRequest): ObservableInput<TRequest>;
    /**
     * Prepares the response by passing it through the `responseHandler.process()` method.
     * This method is an implementation detail of the `_fetch$()` method, and is not part of the public API.
     * It takes a `TResponse` object as input, which represents the response data, and returns an `ObservableInput<TResponse>`,
     * which can be used to further process the response before it is returned.
     *
     * @param response The response data to be processed.
     * @returns An `ObservableInput<TResponse>` that represents the processed response.
     */
    protected _prepareResponse(response: TResponse): ObservableInput<TResponse>;
    /**
     * Resolves the full URL for a given path by combining it with the base URL.
     * This is a protected method and is an implementation detail of the `HttpClient` class.
     *
     * @param path - The path to be resolved.
     * @returns The full URL for the given path.
     */
    protected _resolveUrl(path: string): string;
}
