import { z } from 'zod';
/**
 * Validates that the provided HTTP method string is in uppercase.
 *
 * @link https://www.rfc-editor.org/rfc/rfc7231#section-4.1
 */
export declare const requestMethodCasing: () => z.ZodType<string>;
/**
 * Creates a Zod enum schema for HTTP request methods.
 *
 * The schema validates that the value is one of the standard HTTP methods:
 * 'GET', 'POST', 'PUT', 'DELETE', 'PATCH', 'OPTIONS', 'HEAD', 'CONNECT', 'TRACE'.
 *
 * If the validation fails, a custom error message is returned, indicating the
 * expected methods and the received value. The error message also references
 * RFC 2616 for more information.
 *
 * @link https://www.w3.org/Protocols/rfc2616/rfc2616-sec9.html
 */
export declare const requestMethodVerb: () => z.ZodEnum<["GET", "POST", "PUT", "DELETE", "PATCH", "OPTIONS", "HEAD", "CONNECT", "TRACE"]>;
export declare const requestMethod: () => z.ZodPipeline<z.ZodType<string, z.ZodTypeDef, string>, z.ZodEnum<["GET", "POST", "PUT", "DELETE", "PATCH", "OPTIONS", "HEAD", "CONNECT", "TRACE"]>>;
/**
 * Schema for validating the initialization options of a request.
 *
 * @link https://developer.mozilla.org/en-US/docs/Web/API/Request/Request
 *
 * This schema is used to ensure that the request options conform to the expected structure and types.
 */
export declare const requestInitSchema: z.ZodObject<{
    attributionReporting: z.ZodOptional<z.ZodObject<{
        eventSourceEligible: z.ZodOptional<z.ZodBoolean>;
        triggerEligible: z.ZodOptional<z.ZodBoolean>;
    }, "strip", z.ZodTypeAny, {
        eventSourceEligible?: boolean | undefined;
        triggerEligible?: boolean | undefined;
    }, {
        eventSourceEligible?: boolean | undefined;
        triggerEligible?: boolean | undefined;
    }>>;
    body: z.ZodOptional<z.ZodUnion<[z.ZodString, z.ZodType<Blob, z.ZodTypeDef, Blob>, z.ZodType<ArrayBuffer, z.ZodTypeDef, ArrayBuffer>, z.ZodType<FormData, z.ZodTypeDef, FormData>, z.ZodType<URLSearchParams, z.ZodTypeDef, URLSearchParams>, z.ZodType<ReadableStream<unknown>, z.ZodTypeDef, ReadableStream<unknown>>]>>;
    browsingTopics: z.ZodOptional<z.ZodBoolean>;
    cache: z.ZodOptional<z.ZodEnum<["default", "no-store", "reload", "no-cache", "force-cache", "only-if-cached"]>>;
    credentials: z.ZodOptional<z.ZodEnum<["omit", "same-origin", "include"]>>;
    headers: z.ZodUnion<[z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodString>>, z.ZodType<Headers, z.ZodTypeDef, Headers>]>;
    integrity: z.ZodOptional<z.ZodString>;
    keepalive: z.ZodOptional<z.ZodBoolean>;
    method: z.ZodOptional<z.ZodPipeline<z.ZodType<string, z.ZodTypeDef, string>, z.ZodEnum<["GET", "POST", "PUT", "DELETE", "PATCH", "OPTIONS", "HEAD", "CONNECT", "TRACE"]>>>;
    mode: z.ZodOptional<z.ZodEnum<["same-origin", "cors", "no-cors", "navigate", "websocket"]>>;
    priority: z.ZodOptional<z.ZodEnum<["low", "high", "auto"]>>;
    redirect: z.ZodOptional<z.ZodEnum<["follow", "error", "manual"]>>;
    referrer: z.ZodOptional<z.ZodString>;
    referrerPolicy: z.ZodOptional<z.ZodEnum<["no-referrer", "no-referrer-when-downgrade", "origin", "origin-when-cross-origin", "same-origin", "strict-origin", "strict-origin-when-cross-origin", "unsafe-url"]>>;
    signal: z.ZodOptional<z.ZodType<AbortSignal, z.ZodTypeDef, AbortSignal>>;
}, "strip", z.ZodTypeAny, {
    body?: string | Blob | ArrayBuffer | FormData | URLSearchParams | ReadableStream<unknown> | undefined;
    cache?: "default" | "no-cache" | "force-cache" | "no-store" | "only-if-cached" | "reload" | undefined;
    credentials?: "include" | "omit" | "same-origin" | undefined;
    headers?: Headers | Record<string, string> | undefined;
    integrity?: string | undefined;
    keepalive?: boolean | undefined;
    method?: "GET" | "POST" | "PUT" | "DELETE" | "PATCH" | "OPTIONS" | "HEAD" | "CONNECT" | "TRACE" | undefined;
    mode?: "cors" | "same-origin" | "navigate" | "no-cors" | "websocket" | undefined;
    priority?: "auto" | "high" | "low" | undefined;
    redirect?: "error" | "follow" | "manual" | undefined;
    referrer?: string | undefined;
    referrerPolicy?: "same-origin" | "no-referrer" | "no-referrer-when-downgrade" | "origin" | "origin-when-cross-origin" | "strict-origin" | "strict-origin-when-cross-origin" | "unsafe-url" | undefined;
    signal?: AbortSignal | undefined;
    attributionReporting?: {
        eventSourceEligible?: boolean | undefined;
        triggerEligible?: boolean | undefined;
    } | undefined;
    browsingTopics?: boolean | undefined;
}, {
    body?: string | Blob | ArrayBuffer | FormData | URLSearchParams | ReadableStream<unknown> | undefined;
    cache?: "default" | "no-cache" | "force-cache" | "no-store" | "only-if-cached" | "reload" | undefined;
    credentials?: "include" | "omit" | "same-origin" | undefined;
    headers?: Headers | Record<string, string> | undefined;
    integrity?: string | undefined;
    keepalive?: boolean | undefined;
    method?: string | undefined;
    mode?: "cors" | "same-origin" | "navigate" | "no-cors" | "websocket" | undefined;
    priority?: "auto" | "high" | "low" | undefined;
    redirect?: "error" | "follow" | "manual" | undefined;
    referrer?: string | undefined;
    referrerPolicy?: "same-origin" | "no-referrer" | "no-referrer-when-downgrade" | "origin" | "origin-when-cross-origin" | "strict-origin" | "strict-origin-when-cross-origin" | "unsafe-url" | undefined;
    signal?: AbortSignal | undefined;
    attributionReporting?: {
        eventSourceEligible?: boolean | undefined;
        triggerEligible?: boolean | undefined;
    } | undefined;
    browsingTopics?: boolean | undefined;
}>;
/**
 * Schema for validating fetch request configurations.
 *
 * This schema extends the `requestInitSchema` and adds additional properties:
 * - `uri`: A required string representing the URI of the request.
 * - `path`: An optional string representing the path of the request.
 */
export declare const fetchRequestSchema: z.ZodObject<{
    attributionReporting: z.ZodOptional<z.ZodObject<{
        eventSourceEligible: z.ZodOptional<z.ZodBoolean>;
        triggerEligible: z.ZodOptional<z.ZodBoolean>;
    }, "strip", z.ZodTypeAny, {
        eventSourceEligible?: boolean | undefined;
        triggerEligible?: boolean | undefined;
    }, {
        eventSourceEligible?: boolean | undefined;
        triggerEligible?: boolean | undefined;
    }>>;
    body: z.ZodOptional<z.ZodUnion<[z.ZodString, z.ZodType<Blob, z.ZodTypeDef, Blob>, z.ZodType<ArrayBuffer, z.ZodTypeDef, ArrayBuffer>, z.ZodType<FormData, z.ZodTypeDef, FormData>, z.ZodType<URLSearchParams, z.ZodTypeDef, URLSearchParams>, z.ZodType<ReadableStream<unknown>, z.ZodTypeDef, ReadableStream<unknown>>]>>;
    browsingTopics: z.ZodOptional<z.ZodBoolean>;
    cache: z.ZodOptional<z.ZodEnum<["default", "no-store", "reload", "no-cache", "force-cache", "only-if-cached"]>>;
    credentials: z.ZodOptional<z.ZodEnum<["omit", "same-origin", "include"]>>;
    headers: z.ZodUnion<[z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodString>>, z.ZodType<Headers, z.ZodTypeDef, Headers>]>;
    integrity: z.ZodOptional<z.ZodString>;
    keepalive: z.ZodOptional<z.ZodBoolean>;
    method: z.ZodOptional<z.ZodPipeline<z.ZodType<string, z.ZodTypeDef, string>, z.ZodEnum<["GET", "POST", "PUT", "DELETE", "PATCH", "OPTIONS", "HEAD", "CONNECT", "TRACE"]>>>;
    mode: z.ZodOptional<z.ZodEnum<["same-origin", "cors", "no-cors", "navigate", "websocket"]>>;
    priority: z.ZodOptional<z.ZodEnum<["low", "high", "auto"]>>;
    redirect: z.ZodOptional<z.ZodEnum<["follow", "error", "manual"]>>;
    referrer: z.ZodOptional<z.ZodString>;
    referrerPolicy: z.ZodOptional<z.ZodEnum<["no-referrer", "no-referrer-when-downgrade", "origin", "origin-when-cross-origin", "same-origin", "strict-origin", "strict-origin-when-cross-origin", "unsafe-url"]>>;
    signal: z.ZodOptional<z.ZodType<AbortSignal, z.ZodTypeDef, AbortSignal>>;
} & {
    uri: z.ZodString;
    path: z.ZodOptional<z.ZodString>;
}, "strip", z.ZodTypeAny, {
    uri: string;
    body?: string | Blob | ArrayBuffer | FormData | URLSearchParams | ReadableStream<unknown> | undefined;
    path?: string | undefined;
    cache?: "default" | "no-cache" | "force-cache" | "no-store" | "only-if-cached" | "reload" | undefined;
    credentials?: "include" | "omit" | "same-origin" | undefined;
    headers?: Headers | Record<string, string> | undefined;
    integrity?: string | undefined;
    keepalive?: boolean | undefined;
    method?: "GET" | "POST" | "PUT" | "DELETE" | "PATCH" | "OPTIONS" | "HEAD" | "CONNECT" | "TRACE" | undefined;
    mode?: "cors" | "same-origin" | "navigate" | "no-cors" | "websocket" | undefined;
    priority?: "auto" | "high" | "low" | undefined;
    redirect?: "error" | "follow" | "manual" | undefined;
    referrer?: string | undefined;
    referrerPolicy?: "same-origin" | "no-referrer" | "no-referrer-when-downgrade" | "origin" | "origin-when-cross-origin" | "strict-origin" | "strict-origin-when-cross-origin" | "unsafe-url" | undefined;
    signal?: AbortSignal | undefined;
    attributionReporting?: {
        eventSourceEligible?: boolean | undefined;
        triggerEligible?: boolean | undefined;
    } | undefined;
    browsingTopics?: boolean | undefined;
}, {
    uri: string;
    body?: string | Blob | ArrayBuffer | FormData | URLSearchParams | ReadableStream<unknown> | undefined;
    path?: string | undefined;
    cache?: "default" | "no-cache" | "force-cache" | "no-store" | "only-if-cached" | "reload" | undefined;
    credentials?: "include" | "omit" | "same-origin" | undefined;
    headers?: Headers | Record<string, string> | undefined;
    integrity?: string | undefined;
    keepalive?: boolean | undefined;
    method?: string | undefined;
    mode?: "cors" | "same-origin" | "navigate" | "no-cors" | "websocket" | undefined;
    priority?: "auto" | "high" | "low" | undefined;
    redirect?: "error" | "follow" | "manual" | undefined;
    referrer?: string | undefined;
    referrerPolicy?: "same-origin" | "no-referrer" | "no-referrer-when-downgrade" | "origin" | "origin-when-cross-origin" | "strict-origin" | "strict-origin-when-cross-origin" | "unsafe-url" | undefined;
    signal?: AbortSignal | undefined;
    attributionReporting?: {
        eventSourceEligible?: boolean | undefined;
        triggerEligible?: boolean | undefined;
    } | undefined;
    browsingTopics?: boolean | undefined;
}>;
