import type { Observable } from 'rxjs';
import type { IProcessOperators, ProcessOperator } from './types';
/**
 * ProcessOperators class manages a collection of process operators
 * and provides methods to add, set, get, and process these operators.
 * It implements the IProcessOperators interface for type T.
 *
 * @template T The type of data that the process operators work with
 */
export declare class ProcessOperators<T> implements IProcessOperators<T> {
    /**
     * A record of process operators keyed by a string.
     * This property is used to store and manage the collection of process operators
     * that are used by the `ProcessOperators` class.
     */
    protected _operators: Record<string, ProcessOperator<T>>;
    /**
     * Accessor for the collection of process operators.
     * @returns The record of process operators.
     */
    get operators(): Record<string, ProcessOperator<T>>;
    /**
     * Constructs a new instance of the ProcessOperators class.
     * @param operators - An optional object containing process operators.
     *                    It can be either an instance of IProcessOperators<T> or a record of string keys and ProcessOperator<T> values.
     */
    constructor(operators?: IProcessOperators<T> | Record<string, ProcessOperator<T>>);
    /**
     * Adds a new operator to the collection.
     * @param key The key under which the operator is stored.
     * @param operator The operator to be added.
     * @returns The instance of ProcessOperators for chaining.
     * @throws Error if an operator with the same key already exists.
     */
    add(key: string, operator: ProcessOperator<T>): ProcessOperators<T>;
    /**
     * Sets or updates an operator in the collection.
     * @param key The key under which the operator is stored.
     * @param operator The operator to be set.
     * @returns The instance of ProcessOperators for chaining.
     */
    set(key: string, operator: ProcessOperator<T>): ProcessOperators<T>;
    /**
     * Removes an operator from the collection by its key.
     *
     * @param key - The key of the operator to remove.
     * @returns The current instance of `ProcessOperators` for method chaining.
     */
    remove(key: string): ProcessOperators<T>;
    /**
     * Retrieves an operator from the collection by its key.
     * @param key The key of the operator to retrieve.
     * @returns The retrieved operator.
     */
    get(key: string): ProcessOperator<T>;
    /**
     * Processes an input request through the chain of operators.
     * @param request The request to be processed.
     * @returns An Observable of the processed request.
     */
    process(request: T): Observable<T>;
}
