import type { ResponseSelector } from '../client/types.js';
/**
 * A type representing a function that parses a string into a specific data type.
 *
 * @typeParam TData - The type of the parsed data. Defaults to `unknown` if not specified.
 * @param data - The string input to be parsed.
 * @returns The parsed data of type `TData`.
 */
export type DataParser<TData = unknown> = (data: string) => TData;
/**
 * Represents a Server-Sent Event (SSE) with optional fields.
 */
export type ServerSentEvent<TData = unknown> = {
    id?: string;
    event?: string;
    data?: TData;
    retry?: string;
};
/**
 * Options for configuring the SSE (Server-Sent Events) selector.
 */
export type SseSelectorOptions<TData = unknown> = {
    dataParser?: DataParser<TData>;
    /**
     * A string or an array of strings specifying the events to filter.
     * Only events matching the filter will be processed. If not provided,
     * all events will be processed.
     */
    eventFilter?: string | string[];
    /**
     * A boolean indicating whether to skip processing heartbeat events.
     * Defaults to `false` if not specified.
     */
    skipHeartbeats?: boolean;
    /**
     * An `AbortSignal` that can be used to abort the SSE operation.
     * Useful for managing the lifecycle of the SSE connection.
     */
    abortSignal?: AbortSignal | null;
};
/**
 * A type alias for selecting and transforming Server-Sent Events (SSE) from an HTTP response.
 *
 * @template TData - The type of the data contained within the Server-Sent Event. Defaults to `unknown`.
 * @template TResponse - The type of the HTTP response object. Defaults to `Response`.
 *
 * This selector is used to process and extract `ServerSentEvent<TData>` objects
 * from an HTTP `Response` object, enabling custom handling of SSE data streams.
 */
export type SseSelector<TData = unknown, TResponse extends Response = Response> = ResponseSelector<ServerSentEvent<TData>, TResponse>;
/**
 * Transforms an SSE response into an Observable stream of parsed events.
 * @param response - The HTTP response with Content-Type: text/event-stream.
 * @param options - Optional configuration for event filtering and heartbeat handling.
 * @param options.eventFilter - Filter events by type (single string or array).
 * @param options.skipHeartbeats - Skip empty/heartbeat events if true.
 * @param options.dataParser - Custom parser for the data field.
 * @param options.abortSignal - Abort signal to cancel the stream.
 * @returns An Observable emitting parsed ServerSentEvent objects.
 * @throws ServerSentEventResponseError if response is invalid or stream fails.
 */
export declare const createSseSelector: <TData = unknown, TResponse extends Response = Response>(options?: SseSelectorOptions<TData>) => SseSelector<TData, TResponse>;
export default createSseSelector;
