import { HttpClientMsal } from './lib/client';
import { type IHttpClientConfigurator, type HttpClientOptions } from './configurator';
import { type IHttpClientProvider } from './provider';
import type { Module, ModuleConfigType, IModuleConfigurator } from '@equinor/fusion-framework-module';
import type { MsalModule } from '@equinor/fusion-framework-module-msal';
/**
 * Defines the type for the HTTP module, which includes:
 * - The module name: 'http'
 * - The type of the HTTP client provider, which is `IHttpClientProvider`
 * - The type of the HTTP client configurator, which is `IHttpClientConfigurator`
 */
export type HttpModule = Module<'http', IHttpClientProvider, IHttpClientConfigurator>;
/**
 * Defines the type for the HTTP module with MSAL authentication.
 *
 * This type represents the module configuration for the HTTP module, which includes:
 * - The module name: 'http'
 * - The type of the HTTP client provider, which is `IHttpClientProvider<HttpClientMsal>`
 * - The type of the HTTP client configurator, which is `IHttpClientConfigurator<HttpClientMsal>`
 * - The list of required modules, which includes the `MsalModule`
 */
export type HttpMsalModule = Module<'http', IHttpClientProvider<HttpClientMsal>, IHttpClientConfigurator<HttpClientMsal>, [
    MsalModule
]>;
/**
 * HTTP module with MSAL authentication.
 */
export declare const module: HttpMsalModule;
/**
 * Configures the HTTP module with MSAL authentication.
 */
export declare const configureHttp: <TRef = unknown>(configure: (config: ModuleConfigType<HttpMsalModule>, ref?: TRef) => void) => IModuleConfigurator<HttpMsalModule, TRef>;
/**
 * Configures the HTTP client with MSAL authentication.
 *
 * This function creates a module configurator that can be used to configure the HTTP module
 * with MSAL authentication. The configurator takes a name and a set of HTTP client options,
 * and returns a module configurator that can be used to configure the HTTP module.
 *
 * @param name - The name of the HTTP client configuration.
 * @param args - The HTTP client options, including the MSAL configuration.
 * @returns A module configurator that can be used to configure the HTTP module.
 */
export declare const configureHttpClient: <TRef = unknown>(name: string, args: HttpClientOptions<HttpClientMsal>) => IModuleConfigurator<HttpMsalModule, TRef>;
/**
 * Declares a module named '@equinor/fusion-framework-module' that contains an interface named 'Modules' with a property 'http' of type 'HttpMsalModule'.
 */
declare module '@equinor/fusion-framework-module' {
    interface Modules {
        http: HttpMsalModule;
    }
}
export default module;
